package br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Timer;
import java.util.TimerTask;

import br.ime.usp.mac499.bluetooth.emulator.device.behavior.NFCBehavior;
import br.ime.usp.mac499.nfc.impl.NFCTouchNotify;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Simulation of NFC touch for NFC-capable emulated devices.
 * 
 * The NFC touch can happen after an interval.
 * 
 * @author Rodolpho Atoji
 */
public class NFCTouchBehavior implements NFCBehavior {

	private final String macAddress;

	private final long delayBeforeTouch;
	
	private Timer timer;

	/**
	 * Build a behavior, to happen after the specified time.
	 * 
	 * @param macAddress Device MAC address.
	 * @param delayBeforeTouch Time in milliseconds, to wait before touching
	 * the NFC reader.
	 * @throws RemoteException
	 * @throws NotBoundException
	 */
	public NFCTouchBehavior(String macAddress, long delayBeforeTouch)
			throws RemoteException, NotBoundException {
		super();
		this.delayBeforeTouch = delayBeforeTouch;
		this.macAddress = macAddress;
	}

	@Override
	public void initialize() {
		this.timer = new Timer();
		TimerTask timerTask = new DelayedNFCTouchTask();
		timer.schedule(timerTask, delayBeforeTouch);
	}

	private void doNFCTouch() {
		try {
			Registry registry = LocateRegistry.getRegistry("localhost");
			NFCTouchNotify notify = (NFCTouchNotify) registry
					.lookup("NFCTouch");
			notify.notifyTouch(macAddress, "btgoep://" + macAddress
					+ ":1;authenticate=false;encrypt=false;master=false");
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotBoundException e) {
			e.printStackTrace();
		}
	}
	
	private final class DelayedNFCTouchTask extends TimerTask {

		@Override
		public void run() {
			doNFCTouch();
		}
		
	}

}
