package br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl;

import java.util.Random;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.LocalDevice;

import com.intel.bluetooth.BlueCoveImpl;

import br.ime.usp.mac499.bluetooth.emulator.device.behavior.DiscoveryBehavior;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Makes an emulated device continuously switch between visible and hidden 
 * to the discovery process.
 * 
 * Can be used to simulate RF interferences in the environment, or a
 * device which is in the border of Bluetooth range area, being sometimes
 * visible, sometimes not.
 * 
 * @author Rodolpho Atoji
 */
public class RandomDiscoveryBehavior implements DiscoveryBehavior {

	private final int intervalSeconds;

	private final Random random;
	
	private Object bluetoothStack;

	/**
	 * The specified interval is used to determine the period in which
	 * the device will be in discoverable or hidden mode.
	 * 
	 * An amount of time is uniformly chosen in the interval specified,
	 * for each period of visibility.
	 * 
	 * @param intervalSeconds Maximum seconds to wait between visibility
	 * intervals.
	 */
	public RandomDiscoveryBehavior(int intervalSeconds) {
		this.intervalSeconds = intervalSeconds;
		this.random = new Random();
	}

	@Override
	public void initialize() throws BluetoothStateException {
		this.bluetoothStack = BlueCoveImpl.getCurrentThreadBluetoothStackID();
		
		MakeVisibleTask visibleTask = new MakeVisibleTask();
		Thread visibleThread = new Thread(visibleTask);
		visibleThread.start();
	}

	private final class MakeVisibleTask implements Runnable {

		@Override
		public void run() {
			long nextInterval;
			
			try {
				BlueCoveImpl.setThreadBluetoothStackID(bluetoothStack);
				LocalDevice.getLocalDevice().setDiscoverable(
						DiscoveryAgent.NOT_DISCOVERABLE);

				while (true) {
					nextInterval = getNextInterval();
					Thread.sleep(nextInterval);
					
					LocalDevice.getLocalDevice().setDiscoverable(
							DiscoveryAgent.GIAC);
					
					nextInterval = getNextInterval();
					Thread.sleep(nextInterval);
					
					LocalDevice.getLocalDevice().setDiscoverable(
							DiscoveryAgent.NOT_DISCOVERABLE);
				}
			} catch (BluetoothStateException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		private long getNextInterval() {
			int nextInterval = random.nextInt(intervalSeconds);
			return nextInterval * 1000;
		}

	}

}
