package br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl;

import java.util.Timer;
import java.util.TimerTask;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.LocalDevice;

import com.intel.bluetooth.BlueCoveImpl;

import br.ime.usp.mac499.bluetooth.emulator.device.behavior.DiscoveryBehavior;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Makes a device discoverable to the discovery process.
 * 
 * The behavior can be configured to set the device as discoverable
 * after some time, which can be used to simulate devices arriving at
 * different times.
 * 
 * @author Rodolpho Atoji
 */
public class VisibleDiscoveryBehavior implements DiscoveryBehavior {

	protected final long visibleAfterMillis;

	protected Timer timer;

	protected Object bluetoothStack;

	/**
	 * Sets the device as visible.
	 * 
	 * @param visibleAfterMillis Time, in milliseconds, to set device as
	 * visible.
	 */
	public VisibleDiscoveryBehavior(long visibleAfterMillis) {
		super();
		this.visibleAfterMillis = visibleAfterMillis;
	}

	@Override
	public void initialize() throws BluetoothStateException {
		this.bluetoothStack = BlueCoveImpl.getCurrentThreadBluetoothStackID();
		LocalDevice.getLocalDevice().setDiscoverable(
				DiscoveryAgent.NOT_DISCOVERABLE);

		if (visibleAfterMillis > 0) {
			this.timer = new Timer();
			TimerTask timerTask = new DelayedDiscoverableTask();
			timer.schedule(timerTask, visibleAfterMillis);
		} else {
			makeVisible();
		}
	}

	protected void makeVisible() throws BluetoothStateException {
		BlueCoveImpl.setThreadBluetoothStackID(bluetoothStack);
		LocalDevice.getLocalDevice().setDiscoverable(DiscoveryAgent.GIAC);
	}

	protected final class DelayedDiscoverableTask extends TimerTask {

		@Override
		public void run() {
			try {
				makeVisible();
			} catch (BluetoothStateException e) {
				e.printStackTrace();
			}
		}

	}

}
