package br.ime.usp.mac499.bluetooth.emulator.obex;

import java.io.IOException;
import java.io.InputStream;

import javax.obex.Operation;
import javax.obex.ResponseCodes;
import javax.obex.ServerRequestHandler;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Extension of a JSR 82 <code>ServerRequestHandler</code>.
 * 
 * Implements real time transfers delay, based on transfer rate.
 * 
 * @author Rodolpho Atoji
 */
public class GenericServerRequestHandler extends ServerRequestHandler {

	protected final int bytesPerSecond;

	public GenericServerRequestHandler(int bitsPerSecond) {
		super();
		int convertedBytes = bitsPerSecond / 10;

		if (convertedBytes > 0)
			this.bytesPerSecond = convertedBytes;
		else
			this.bytesPerSecond = 1;

	}

	@Override
	public int onPut(Operation op) {
		byte readBytes[] = new byte[bytesPerSecond];

		try {
			InputStream inputStream = op.openInputStream();

			while ((inputStream.read(readBytes) > 0)) {
				Thread.sleep(1000);
			}

			inputStream.close();
			return ResponseCodes.OBEX_HTTP_OK;
		} catch (IOException e) {
			e.printStackTrace();
			return ResponseCodes.OBEX_HTTP_INTERNAL_ERROR;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return ResponseCodes.OBEX_HTTP_INTERNAL_ERROR;
		}
	}

}
