package br.ime.usp.mac499.bluetooth.emulator.obex;

import java.io.IOException;
import java.io.InputStream;
import java.util.Random;

import javax.obex.Operation;
import javax.obex.ResponseCodes;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Extension of JSR 82 <code>ServerRequestHandler</code>.
 * 
 * Implements real time transfer delay and RF channel dropping.
 * 
 * @author Rodolpho Atoji
 */
public class InterruptingServerRequestHandler extends
		GenericServerRequestHandler {

	public InterruptingServerRequestHandler(int bitsPerSecond) {
		super(bitsPerSecond);
	}

	@Override
	public int onPut(Operation op) {
		byte readBytes[] = new byte[bytesPerSecond];
		
		try {
			InputStream inputStream = op.openInputStream();
			
			int abortStep = getAbortStep(op.getLength(), bytesPerSecond);
			
			for (int step = 0; inputStream.read(readBytes) > 0; step++) {
				if (step == abortStep) {
					break;
				}
				Thread.sleep(1000);
			}

			/*
			 * For some reason, just this response code can abort
			 * the transfer... 
			 */
			return ResponseCodes.OBEX_HTTP_UNAUTHORIZED;
		} catch (IOException e) {
			e.printStackTrace();
			return ResponseCodes.OBEX_HTTP_UNAUTHORIZED;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return ResponseCodes.OBEX_HTTP_UNAUTHORIZED;
		}
	}
	
	private int getAbortStep(long contentLength, int bytesPerSecond) {
		long steps = contentLength / bytesPerSecond;
		
		Long longSteps = new Long(steps);
		int intSteps = longSteps.intValue();
		
		if (intSteps == 0)
			intSteps = 1;
		
		Random random = new Random();
		int abortStep = random.nextInt(intSteps);
		
		return abortStep;
	}

}
