package br.ime.usp.mac499.bluetooth.emulator.obex;

import java.util.Random;

import javax.obex.HeaderSet;
import javax.obex.ResponseCodes;
import javax.obex.ServerRequestHandler;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Extension of a JSR 82 <code>ServerRequestHandle</code>.
 * 
 * Implements rejecting an incoming OBEX file push.
 * 
 * Currently the rejection is made after some time chosen uniformly
 * between 0 and 60 seconds.
 * 
 * @author Rodolpho Atoji
 */
public class RejectingServerRequestHandler extends ServerRequestHandler {

	private final static int MAX_RANDOM_REJECT_INTERVAL = 60000;
	
	@Override
	public int onConnect(HeaderSet request, HeaderSet reply) {
		Random random = new Random();
		int randomWaitInterval = random.nextInt(MAX_RANDOM_REJECT_INTERVAL);
		
		try {
			Thread.sleep(randomWaitInterval);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		return ResponseCodes.OBEX_HTTP_INTERNAL_ERROR;
	}

}
