package br.ime.usp.mac499.bluetooth.simulation;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Supported emulated visibility modes.
 * 
 * @author Rodolpho Atoji
 */
public enum EmulatedDiscoverableMode {

	/**
	 * Device is visible to the discovery process.
	 */
	VISIBLE		("visible"),
	
	/**
	 * Device is hidden to the discovery process.
	 */
	HIDDEN		("hidden"),
	
	/**
	 * Device continuously switches between visible and hidden.
	 */
	RANDOM		("random");
	
	private final String modeString;

	private EmulatedDiscoverableMode(String modeString) {
		this.modeString = modeString;
	}

	public String getModeString() {
		return modeString;
	}
	
	public static EmulatedDiscoverableMode getMapping(String modeString) {
		for (EmulatedDiscoverableMode mode : values()) {
			if (mode.getModeString().equals(modeString))
				return mode;
		}
		
		return null;
	}
	
}
