package br.ime.usp.mac499.bluetooth.simulation;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Supported simulated incoming transfer behaviors.
 * 
 * @author Rodolpho Atoji
 */
public enum IncomingTransferBehavior {
	
	/**
	 * Promptly accepts incoming file transfer.
	 */
	ACCEPT		("accept"),
	
	/**
	 * Rejects file transfer after a while.
	 */
	REJECT		("reject"),
	
	/**
	 * Drops the file transfer after a while.
	 */
	CORRUPT		("corrupt");
	
	private final String behaviorString;

	private IncomingTransferBehavior(String behaviorString) {
		this.behaviorString = behaviorString;
	}

	public String getBehaviorString() {
		return behaviorString;
	}
	
	public static IncomingTransferBehavior getMapping(String behaviorString) {
		for (IncomingTransferBehavior behavior : values()) {
			if (behavior.getBehaviorString().equals(behaviorString))
				return behavior;
		}
		
		return null;
	}

}
