package br.ime.usp.mac499.bluetooth.simulation;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * This class is not specially useful, but it can illustrate the
 * process of generation of simulation files.
 * 
 * @author Rodolpho Atoji
 */
public class SimulationScenarioFileGenerator {

	private static final String BASE_PATH_SIM1 = "simulation/sim1";

	private static final String BASE_PATH_SIM2 = "simulation/sim2";

	private static final String BASE_PATH_SIM3 = "simulation/sim3";

	private static final String BASE_PATH_SIM4 = "simulation/sim4";

	private static final String BASE_PATH_SIM5 = "simulation/sim5";

	private static final String BASE_PATH_SIM6 = "simulation/sim6";

	private static final String BASE_PATH_SIM7 = "simulation/sim7";

	private static final String BASE_PATH_SIM8 = "simulation/sim8";
	
	private static final String SDP_ENABLED = "%SDP_ENABLED%";
	private static final String INCOMING_TRANSFER = "%INCOMING_TRANSFER%";
	private static final String NFC_TOUCH_AFTER = "%NFC_TOUCH_AFTER%";
	private static final String NFC_CAPABLE = "%NFC_CAPABLE%";
	private static final String TRANSFER_RATE = "%TRANSFER_RATE%";
	private static final String RANDOM_DISCOVERY = "%RANDOM_DISCOVERY%";
	private static final String DISCOVERABLE_AFTER = "%DISCOVERABLE_AFTER%";
	private static final String DISCOVERABLE_MODE = "%DISCOVERABLE_MODE%";
	private static final String MAC_ADDRESS = "%MAC_ADDRESS%";
	private static final String ID = "%ID%";

	private final static String DEVICE_TEMPLATE = "device.%ID%=%MAC_ADDRESS%\n"
			+ "device.%ID%.discoverable_mode=%DISCOVERABLE_MODE%\n"
			+ "device.%ID%.discoverable_after=%DISCOVERABLE_AFTER%\n"
			+ "device.%ID%.random_discovery=%RANDOM_DISCOVERY%\n"
			+ "device.%ID%.transfer_rate=%TRANSFER_RATE%\n"
			+ "device.%ID%.nfc_capable=%NFC_CAPABLE%\n"
			+ "device.%ID%.nfc_touch_after=%NFC_TOUCH_AFTER%\n"
			+ "device.%ID%.incoming_transfer=%INCOMING_TRANSFER%\n"
			+ "device.%ID%.sdp_enabled=%SDP_ENABLED%\n";

	public static void main(String[] args) {
		generate10DevicesSim1();
		generate25DevicesSim1();
		generate50DevicesSim1();
		generate100DevicesSim1();

		generate10DevicesSim2();
		generate25DevicesSim2();
		generate50DevicesSim2();
		generate100DevicesSim2();

		generate10DevicesSim3();
		generate25DevicesSim3();
		generate50DevicesSim3();
		generate100DevicesSim3();

		generate10DevicesSim4();
		generate25DevicesSim4();
		generate50DevicesSim4();
		generate100DevicesSim4();

		generate10DevicesSim5();
		generate25DevicesSim5();
		generate50DevicesSim5();
		generate100DevicesSim5();

		generateSim6();
		generateSim7();
		generateSim8();
	}

	private static void generateSim8() {
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_10d.properties", 10,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_25d.properties", 25,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_50d.properties", 50,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_100d.properties", 100,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_150d.properties", 150,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM8 + "/sim8_200d.properties", 200,
				500000, true, 0.25, 2, 0.25, 0.25, 0.25);
	}

	private static void generateSim7() {
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_10d.properties", 10,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_25d.properties", 25,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_50d.properties", 50,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_100d.properties", 100,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_150d.properties", 150,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM7 + "/sim7_200d.properties", 200,
				500000, false, 0.25, 2, 0.25, 0.25, 0.25);
	}

	private static void generateSim6() {
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_10d.properties", 10,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_25d.properties", 25,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_50d.properties", 50,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_100d.properties", 100,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_150d.properties", 150,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
		generateMixedScenario(BASE_PATH_SIM6 + "/sim6_200d.properties", 200,
				500000, false, 0, 0, 0.25, 0.25, 0.25);
	}

	private static void generate100DevicesSim1() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_100d_00r.properties", 100, 500000, 0.00,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_100d_10r.properties", 100, 500000, 0.10,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_100d_25r.properties", 100, 500000, 0.25,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_100d_50r.properties", 100, 500000, 0.50,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_100d_75r.properties", 100, 500000, 0.75,
				IncomingTransferBehavior.REJECT, false, 0, 0);
	}

	private static void generate50DevicesSim1() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_50d_00r.properties", 50, 500000, 0.00,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_50d_10r.properties", 50, 500000, 0.10,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_50d_25r.properties", 50, 500000, 0.25,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_50d_50r.properties", 50, 500000, 0.50,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_50d_75r.properties", 50, 500000, 0.75,
				IncomingTransferBehavior.REJECT, false, 0, 0);
	}

	private static void generate25DevicesSim1() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_25d_00r.properties", 25, 500000, 0.00,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_25d_10r.properties", 25, 500000, 0.10,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_25d_25r.properties", 25, 500000, 0.25,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_25d_50r.properties", 25, 500000, 0.50,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_25d_75r.properties", 25, 500000, 0.75,
				IncomingTransferBehavior.REJECT, false, 0, 0);
	}

	private static void generate10DevicesSim1() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_10d_00r.properties", 10, 500000, 0.00,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_10d_10r.properties", 10, 500000, 0.10,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_10d_25r.properties", 10, 500000, 0.25,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_10d_50r.properties", 10, 500000, 0.50,
				IncomingTransferBehavior.REJECT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM1
				+ "/sim1_10d_75r.properties", 10, 500000, 0.75,
				IncomingTransferBehavior.REJECT, false, 0, 0);
	}

	private static void generate100DevicesSim2() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_100d_00e.properties", 100, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_100d_10e.properties", 100, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_100d_25e.properties", 100, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_100d_50e.properties", 100, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_100d_75e.properties", 100, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
	}

	private static void generate50DevicesSim2() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_50d_00e.properties", 50, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_50d_10e.properties", 50, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_50d_25e.properties", 50, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_50d_50e.properties", 50, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_50d_75e.properties", 50, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
	}

	private static void generate25DevicesSim2() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim1_25d_00e.properties", 25, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim1_25d_10e.properties", 25, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim1_25d_25e.properties", 25, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim1_25d_50e.properties", 25, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim1_25d_75e.properties", 25, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
	}

	private static void generate10DevicesSim2() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_10d_00e.properties", 10, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_10d_10e.properties", 10, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_10d_25e.properties", 10, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_10d_50e.properties", 10, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM2
				+ "/sim2_10d_75e.properties", 10, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, false, 0, 0);
	}

	private static void generate100DevicesSim3() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_100d_00r.properties", 100, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_100d_10r.properties", 100, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_100d_25r.properties", 100, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_100d_50r.properties", 100, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_100d_75r.properties", 100, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
	}

	private static void generate50DevicesSim3() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_50d_00r.properties", 50, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_50d_10r.properties", 50, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_50d_25r.properties", 50, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_50d_50r.properties", 50, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.3, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_50d_75r.properties", 50, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
	}

	private static void generate25DevicesSim3() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_25d_00r.properties", 25, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_25d_10r.properties", 25, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_25d_25r.properties", 25, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_25d_50r.properties", 25, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_25d_75r.properties", 25, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
	}

	private static void generate10DevicesSim3() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_10d_00r.properties", 10, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_10d_10r.properties", 10, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_10d_25r.properties", 10, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_10d_50r.properties", 10, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM3
				+ "/sim3_10d_75r.properties", 10, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
	}

	private static void generate100DevicesSim4() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_100d_00e.properties", 100, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_100d_10e.properties", 100, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_100d_25e.properties", 100, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_100d_50e.properties", 100, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_100d_75e.properties", 100, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
	}

	private static void generate50DevicesSim4() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_50d_00e.properties", 50, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_50d_10e.properties", 50, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_50d_25e.properties", 50, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_50d_50e.properties", 50, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_50d_75e.properties", 50, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
	}

	private static void generate25DevicesSim4() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_25d_00e.properties", 25, 500000, 0.00,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_25d_10e.properties", 25, 500000, 0.10,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_25d_25e.properties", 25, 500000, 0.25,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_25d_50e.properties", 25, 500000, 0.50,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_25d_75e.properties", 25, 500000, 0.75,
				IncomingTransferBehavior.CORRUPT, true, 0.2, 5);
	}

	private static void generate10DevicesSim4() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_10d_00e.properties", 10, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_10d_10e.properties", 10, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_10d_25e.properties", 10, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_10d_50e.properties", 10, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM4
				+ "/sim4_10d_75e.properties", 10, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 5);
	}

	private static void generate100DevicesSim5() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_100d_00r.properties", 100, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_100d_10r.properties", 100, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_100d_25r.properties", 100, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_100d_50r.properties", 100, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_100d_75r.properties", 100, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
	}

	private static void generate50DevicesSim5() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_50d_00r.properties", 50, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_50d_10r.properties", 50, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_50d_25r.properties", 50, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_50d_50r.properties", 50, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_50d_75r.properties", 50, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
	}

	private static void generate25DevicesSim5() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_25d_00r.properties", 25, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_25d_10r.properties", 25, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_25d_25r.properties", 25, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_25d_50r.properties", 25, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_25d_75r.properties", 25, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
	}

	private static void generate10DevicesSim5() {
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_10d_00r.properties", 10, 500000, 0.00,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_10d_10r.properties", 10, 500000, 0.10,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_10d_25r.properties", 10, 500000, 0.25,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_10d_50r.properties", 10, 500000, 0.50,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
		generateScenarioWithTransferBehavior(BASE_PATH_SIM5
				+ "/sim5_10d_75r.properties", 10, 500000, 0.75,
				IncomingTransferBehavior.REJECT, true, 0.2, 1);
	}

	public static void generateMixedScenario(String filePath, int devices,
			int transferRate, boolean nfcVisible, double nfcPercent,
			int nfcTouchInterval, double randomVisibleAcceptPercent,
			double randomVisibleRejectPercent,
			double randomVisibleCorruptPercent) {
		StringBuffer buffer = new StringBuffer();

		double nfcDevicesDouble = nfcPercent * devices;
		int nfcDevices = (int) Math.ceil(nfcDevicesDouble);

		double randomAcceptDouble = randomVisibleAcceptPercent * devices;
		int randomAccept = (int) Math.ceil(randomAcceptDouble);

		double randomRejectDouble = randomVisibleRejectPercent * devices;
		int randomReject = (int) Math.ceil(randomRejectDouble);

		double randomCorruptDouble = randomVisibleCorruptPercent * devices;
		int randomCorrupt = (int) Math.ceil(randomCorruptDouble);

		int i = 0;
		int limit = nfcDevices;
		EmulatedDiscoverableMode nfcDiscoverableMode;
		if (nfcVisible)
			nfcDiscoverableMode = EmulatedDiscoverableMode.VISIBLE;
		else
			nfcDiscoverableMode = EmulatedDiscoverableMode.HIDDEN;
		for (int j = nfcTouchInterval; i < limit; i++, j += nfcTouchInterval) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.ACCEPT, true, j,
					nfcDiscoverableMode);
			buffer.append(currentDevice + "\n");
		}

		limit += randomAccept;
		for (; i < limit; i++) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.ACCEPT, false, 0,
					EmulatedDiscoverableMode.RANDOM);
			buffer.append(currentDevice + "\n");
		}

		limit += randomReject;
		for (; i < limit; i++) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.REJECT, false, 0,
					EmulatedDiscoverableMode.RANDOM);
			buffer.append(currentDevice + "\n");
		}

		limit += randomCorrupt;
		for (; i < limit; i++) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.CORRUPT, false, 0,
					EmulatedDiscoverableMode.RANDOM);
			buffer.append(currentDevice + "\n");
		}

		limit = devices;
		for (; i < limit; i++) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.ACCEPT, false, 0,
					EmulatedDiscoverableMode.VISIBLE);
			buffer.append(currentDevice + "\n");
		}

		writeToFile(filePath, buffer.toString());
	}

	public static void generateScenarioWithTransferBehavior(String filePath,
			int devices, int transferRate, double behaviorPercent,
			IncomingTransferBehavior behavior, boolean nfcCapable,
			double nfcPercent, int nfcTouchInterval) {
		StringBuffer buffer = new StringBuffer();

		int i = 0;
		double rejectingDevices = behaviorPercent * devices;
		int rejecting = (int) Math.ceil(rejectingDevices);

		for (; i < rejecting; i++) {
			String currentDevice = generateDevice(transferRate, i, behavior,
					false, 0, EmulatedDiscoverableMode.VISIBLE);
			buffer.append(currentDevice + "\n");
		}

		if (nfcCapable) {
			double nfcDevices = nfcPercent * (devices - rejecting);
			int nfc = (int) Math.ceil(nfcDevices);

			for (int j = nfcTouchInterval; i < (rejecting + nfc); i++, j += nfcTouchInterval) {
				String currentDevice = generateDevice(transferRate, i,
						IncomingTransferBehavior.ACCEPT, true, j,
						EmulatedDiscoverableMode.HIDDEN);
				buffer.append(currentDevice + "\n");
			}
		}

		for (; i < devices; i++) {
			String currentDevice = generateDevice(transferRate, i,
					IncomingTransferBehavior.ACCEPT, false, 0,
					EmulatedDiscoverableMode.VISIBLE);
			buffer.append(currentDevice + "\n");
		}

		writeToFile(filePath, buffer.toString());
	}

	private static void writeToFile(String filePath, String fileContents) {
		File simulationFile = new File(filePath);

		if (simulationFile.exists()) {
			System.err
					.println("[ERROR] File already exists: " + simulationFile);
			return;
		}

		try {
			FileWriter fileWriter = new FileWriter(simulationFile);
			fileWriter.write(fileContents);
			fileWriter.close();
		} catch (IOException e) {
			System.err.println("[ERROR] Cannot write fo file: " + filePath);
		}
	}

	private static String generateDevice(int transferRate, int i,
			IncomingTransferBehavior transferBehavior, boolean nfcCapable,
			int nfcTouchAfter, EmulatedDiscoverableMode discoverableMode) {
		String currentDevice;
		currentDevice = new String(DEVICE_TEMPLATE);
		currentDevice = currentDevice.replaceAll(ID, String.valueOf(i));
		currentDevice = currentDevice.replaceAll(MAC_ADDRESS,
				formatMacAddress(i));

		// if (!nfcCapable) {
		// currentDevice = currentDevice.replaceAll(DISCOVERABLE_MODE,
		// EmulatedDiscoverableMode.VISIBLE.getModeString());
		// } else {
		// currentDevice = currentDevice.replaceAll(DISCOVERABLE_MODE,
		// EmulatedDiscoverableMode.HIDDEN.getModeString());
		// }

		currentDevice = currentDevice.replaceAll(DISCOVERABLE_MODE,
				discoverableMode.getModeString());
		currentDevice = currentDevice.replaceAll(DISCOVERABLE_AFTER, "0");
		currentDevice = currentDevice.replaceAll(RANDOM_DISCOVERY, "15");
		currentDevice = currentDevice.replaceAll(TRANSFER_RATE,
				String.valueOf(transferRate));
		currentDevice = currentDevice.replaceAll(NFC_CAPABLE,
				String.valueOf(nfcCapable));
		currentDevice = currentDevice.replaceAll(NFC_TOUCH_AFTER,
				String.valueOf(nfcTouchAfter));
		currentDevice = currentDevice.replaceAll(INCOMING_TRANSFER,
				transferBehavior.getBehaviorString());
		currentDevice = currentDevice.replaceAll(SDP_ENABLED, "true");
		return currentDevice;
	}

	private static String formatMacAddress(int deviceIndex) {
		NumberFormat numberFormat = new DecimalFormat("000000000000");
		return numberFormat.format(deviceIndex);
	}

}
