package br.ime.usp.mac499.test.bluetooth.emulator;

import java.util.Properties;

import static org.junit.Assert.*;
import org.junit.Test;

import br.ime.usp.mac499.bluetooth.emulator.device.EmulatedAdvertisingDevice;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.DiscoveryBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.NFCBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.ServiceBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.TransferBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.CorruptTransferBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.InvisibleDiscoveryBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.NFCTouchBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.NormalTransferBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.OBEXPushServiceBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.RandomDiscoveryBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.RejectTransferBehavior;
import br.ime.usp.mac499.bluetooth.emulator.device.behavior.impl.VisibleDiscoveryBehavior;
import br.ime.usp.mac499.bluetooth.simulation.PropertiesSimulationScenarioParser;
import br.ime.usp.mac499.bluetooth.simulation.SimulationScenario;

public class EmulatedAdvertisingDeviceFactoryTest {

	@Test
	public void testDiscoverableCorruptSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testDiscoverableNormalSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testDiscoverableRejectSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableCorruptSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableNormalSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableRejectSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testRandomCorruptSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testRandomNormalSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testRandomRejectSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testRandomCorruptSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testRandomNormalSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testRandomRejectSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "random");
		properties.put("device.0.random_discovery", "10");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(RandomDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testInvisibleCorruptSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testInvisibleNormalSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testInvisibleRejectSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testInvisibleCorruptSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testInvisibleNormalSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testInvisibleRejectSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "hidden");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "true");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(OBEXPushServiceBehavior.class, serviceBehavior.getClass());
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(InvisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableCorruptNoSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "false");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testDiscoverableNormalNoSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "false");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}
	
	@Test
	public void testDiscoverableRejectNoSdpNoNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "false");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(nfcBehavior, null);
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableCorruptNoSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "corrupt");
		properties.put("device.0.sdp_enabled", "false");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(CorruptTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableNormalNoSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "accept");
		properties.put("device.0.sdp_enabled", "false");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(NormalTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}

	@Test
	public void testDiscoverableRejectNoSdpNfc() {
		Properties properties = new Properties();
		properties.put("device.0", "001122334455");
		properties.put("device.0.discoverable_mode", "visible");
		properties.put("device.0.transfer_rate", "2500000");
		properties.put("device.0.nfc_capable", "false");
		properties.put("device.0.incoming_transfer", "reject");
		properties.put("device.0.sdp_enabled", "false");
		properties.put("device.0.nfc_capable", "true");
		properties.put("device.0.nfc_touch_after", "10");

		PropertiesSimulationScenarioParser parser = new PropertiesSimulationScenarioParser();
		SimulationScenario scenario = parser.loadScenario(properties);
		EmulatedAdvertisingDevice device = scenario.getDeviceList().get(0);
		
		String macAddress = device.getMacAddress();
		assertEquals(macAddress, "001122334455");
		
		int transferRate = device.getTransferRate();
		assertEquals(transferRate, 2500000);
		
		NFCBehavior nfcBehavior = device.getNfcBehavior();
		assertEquals(NFCTouchBehavior.class, nfcBehavior.getClass());
		
		ServiceBehavior serviceBehavior = device.getServiceBehavior();
		assertEquals(null, serviceBehavior);
		
		TransferBehavior transferBehavior = device.getTransferBehavior();
		assertEquals(RejectTransferBehavior.class, transferBehavior.getClass());
		
		DiscoveryBehavior discoveryBehavior = device.getDiscoveryBehavior();
		assertEquals(VisibleDiscoveryBehavior.class, discoveryBehavior.getClass());
	}


}
