package javax.microedition.contactless.ndef;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class NDEFMessage {

	private final List<NDEFRecord> recordsList;

	private byte[] data;

	public NDEFMessage() {
		this.recordsList = new LinkedList<NDEFRecord>();
	}

	public NDEFMessage(NDEFRecord[] records) {
		this();

		for (int i = 0; i < records.length; i++)
			recordsList.add(records[i]);
	}

	public NDEFMessage(byte[] data, int offset) {
		this();
		this.data = Arrays.copyOfRange(data, offset, data.length - 1);
	}

	public int getNumberOfRecords() {
		return recordsList.size();
	}

	public NDEFRecordType[] getRecordTypes() {
		List<NDEFRecordType> recordTypeList = new LinkedList<NDEFRecordType>();
		
		NDEFRecordType recordType;
		for (NDEFRecord record: recordsList) {
			recordType = record.getRecordType();
			if (recordType != null) {
				if (!recordTypeList.contains(recordType))
					recordTypeList.add(recordType);
			}
		}
		
		NDEFRecordType[] recordTypeArray = new NDEFRecordType[0]; 
		return recordTypeList.toArray(recordTypeArray);
	}

	public NDEFRecord getRecord(int index) {
		return recordsList.get(index);
	}

	public NDEFRecord[] getRecord(NDEFRecordType recordType) {
		List<NDEFRecord> recordList = new LinkedList<NDEFRecord>();
		
		for (NDEFRecord record: recordList) {
			if (record.getRecordType().equals(recordType))
				recordList.add(record);
		}
		
		NDEFRecord[] recordArrays = new NDEFRecord[0];
		return recordList.toArray(recordArrays);
	}

	public NDEFRecord getRecord(byte[] id) {
		for (NDEFRecord record: recordsList) {
			if (Arrays.equals(record.getId(), id))
				return record;
		}
		
		return null;
	}

	public NDEFRecord[] getRecords() {
		NDEFRecord[] recordArray = new NDEFRecord[0];
		return recordsList.toArray(recordArray);
	}

	public void setRecord(int index, NDEFRecord record) {
		recordsList.set(index, record);
	}

	public void appendRecord(NDEFRecord record) {
		recordsList.add(record);
	}

	public void insertRecord(int index, NDEFRecord record) {
		recordsList.add(index, record);
	}

	public void removeRecord(int index) {
		recordsList.remove(index);
	}

	public byte[] toByteArray() {
		return data;
	}

}
