package br.ime.usp.mac499.bluetooth.advertising;

import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Specifies a link in a chain of filters to be applied in the advertising 
 * process.
 * 
 * The filter chain is applied to each target device when advertising, even
 * when retrying.
 * 
 * @author Rodolpho Atoji
 */
public interface AdvertisingDeviceFilterChain {

	/**
	 * Sets the next link in the filter chain.
	 * 
	 * The filters are applied in the order they are set by this method.
	 * 
	 * @param processor
	 */
	public abstract void setNext(AdvertisingDeviceFilterChain processor);
	
	/**
	 * If a link of the chain decides that some device needs to be filtered
	 * out, the filter processing ends on this link. Otherwise, the next
	 * link in the chain is called.
	 * 
	 * @param device Device subject to the filtering.
	 * @return <code>true</code> if the device should be filtered out, 
	 * 	<code>false</code>, otherwise.
	 */
	public abstract boolean shouldFilter(AdvertisingDevice device);
	
}
