package br.ime.usp.mac499.bluetooth.advertising;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import br.ime.usp.mac499.bluetooth.common.BluetoothResources;
import br.ime.usp.mac499.bluetooth.common.config.BluetoothParameters;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Takes care of the advertising process, starting or stopping it.
 * 
 * @author Rodolpho Atoji
 */
public class AdvertisingManager {

	private final RunnableAdvertiser advertiser;
	
	private final BluetoothParameters parameters;
	
	private final ScheduledExecutorService advertisingPool;

	/**
	 * Constructs an advertising manager providing an advertising task to it.
	 * 
	 * This task is submitted to execution on the run of 
	 * <code>startAdvertisng()</code>.
	 * 
	 * @param advertiser Advertising task.
	 */
	public AdvertisingManager(RunnableAdvertiser advertiser) {
		this.advertiser = advertiser;
		this.parameters = BluetoothResources.getParameters();
		this.advertisingPool = Executors.newSingleThreadScheduledExecutor();
	}
	
	/**
	 * Starts advertising process, consuming devices resulting of discovery
	 * process.
	 */
	public void startAdvertising() {
		long advertisingInterval = parameters.getAdvertisingInterval();
		
		advertisingPool.scheduleAtFixedRate(advertiser, 0, advertisingInterval,
				TimeUnit.MILLISECONDS);
	}
	
	/**
	 * Stops advertising process. There's no warranty on the immediate 
	 * stop of advertising to currently advertising devices.
	 */
	public void stopAdvertising() {
		advertisingPool.shutdown();
	}

}
