package br.ime.usp.mac499.bluetooth.advertising;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Available filter operation modes, specified in the file
 * <code>filter.properties</code>.
 * 
 * @author Rodolpho Atoji
 */
public enum FilterMode {

	/**
	 * Even if specified, no devices are filtered out.
	 */
	ALLOW_ALL		("allow_all"),
	
	/**
	 * Even if allowed, all devices are filtered out.
	 */
	DENY_ALL		("deny_all"),
	
	/**
	 * Takes individual actions for each specified target device.
	 */
	DO_FILTER		("do_filter");
	
	private final String relatedProperty;

	private FilterMode(String relatedProperty) {
		this.relatedProperty = relatedProperty;
	}

	/**
	 * @return String property representation of an enum.
	 */
	public String getRelatedProperty() {
		return relatedProperty;
	}
	
	/**
	 * Given a property in <code>filter.properties</code>, map it to
	 * the corresponding enum.
	 * 
	 * @param property String property.
	 * @return Mapped enum to string property.
	 */
	public static FilterMode getMappingForProperty(String property) {
		if (property == null)
			return ALLOW_ALL;
		
		for (FilterMode mapped : values()) {
			if (property.equals(mapped.getRelatedProperty()))
				return mapped;
		}
		
		return ALLOW_ALL;
	}
	
}
