package br.ime.usp.mac499.bluetooth.common;

import br.ime.usp.mac499.bluetooth.common.discovery.DeviceDiscoverer;
import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveryType;
import br.ime.usp.mac499.bluetooth.common.sdp.ServiceDiscoverer;
import br.ime.usp.mac499.bluetooth.io.FilePusher;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * An scheduler of Bluetooth adapters is responsible of submitting tasks
 * to each one according to an arbitrary policy.
 * 
 * Scheduling strategies can fit specific purposes: maximize bandwidth,
 * throughput etc.
 * 
 * @author Rodolpho Atoji
 */
public interface BluetoothAdapterScheduler {

	/**
	 * Schedule a task of the service search process.
	 * 
	 * @param sdpTask Task of the service search process.
	 */
	public void submitServiceDiscoveryTask(ServiceDiscoverer sdpTask);

	/**
	 * Schedule for execution a file transfer.
	 * 
	 * @param pusherTask File transfer task.
	 */
	public void submitFileTransferTask(FilePusher pusherTask);

	/**
	 * Schedule a discovery process.
	 * 
	 * @param discovererTask Discovery process task.
	 * @param discoveryType General or limited.
	 */
	public void registerDiscoveryTask(DeviceDiscoverer discovererTask,
			DiscoveryType discoveryType);

}
