package br.ime.usp.mac499.bluetooth.common;

import br.ime.usp.mac499.bluetooth.common.config.BluetoothParameters;
import br.ime.usp.mac499.bluetooth.common.stack.LocalAdaptersManager;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDeviceManager;
import br.ime.usp.mac499.bluetooth.model.MediaManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Data holder class, for shared objects of the system:
 * 
 * <ul>
 * 	<li>Runtime parameters;</li>
 * 	<li>Monitor for concurrency control;</li>
 * 	<li>Bluetooth adapters manager;</li>
 * 	<li>Advertising devices manager;</li>
 * 	<li>Multimedia contents manager.</li>
 * </ul>
 * 
 * @author Rodolpho Atoji
 */
public class BluetoothResources {

	private static BluetoothParameters parameters;

	private static BluetoothAdapterMonitor monitor;

	private static LocalAdaptersManager localAdaptersManager;

	private static AdvertisingDeviceManager deviceManager;

	private static MediaManager mediaManager;

	public static LocalAdaptersManager getLocalAdaptersManager() {
		return localAdaptersManager;
	}

	public static BluetoothAdapterMonitor getBluetoothAdapterMonitor() {
		return monitor;
	}

	public static BluetoothParameters getParameters() {
		return parameters;
	}

	public static AdvertisingDeviceManager getAdvertisingDeviceManager() {
		return deviceManager;
	}
	
	public static MediaManager getAdvertisingMediaManager() {
		return mediaManager;
	}

	/**
	 * This method should be called in the system initialization, prior to
	 * starting the advertising process.
	 * 
	 * @param localAdaptersManager Bluetooth adapters manager.
	 * @param monitor Concurrent control access monitor.
	 * @param parameters Runtime parameters.
	 * @param deviceManager Advertising devices manager.
	 * @param mediaManager Multimedia content manager.
	 */
	public static void initialize(LocalAdaptersManager localAdaptersManager,
			BluetoothAdapterMonitor monitor, BluetoothParameters parameters,
			AdvertisingDeviceManager deviceManager,
			MediaManager mediaManager) {
		BluetoothResources.localAdaptersManager = localAdaptersManager;
		BluetoothResources.monitor = monitor;
		BluetoothResources.parameters = parameters;
		BluetoothResources.deviceManager = deviceManager;
		BluetoothResources.mediaManager = mediaManager;
	}

}
