package br.ime.usp.mac499.bluetooth.common.config;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Runtime parameters, which define the behavior of the system.
 * 
 * @author Rodolpho Atoji
 */
public interface BluetoothParameters {

	/**
	 * All recognized parameters.
	 */
	final static String[] PARAMETER_LIST = { "discovery.interval",
			"advertising.max_retries", "advertising.max_sucesses",
			"advertising.interval", "bluetooth.max_concurrent",
			"bluetooth.discovery_adapter" };

	/**
	 * Interval between discovery processes.
	 * 
	 * @return Interval between discoveries, in milliseconds.
	 */
	public long getDiscoveryInterval();

	/**
	 * Maximum retries of advertising to a device, counting refusing and
	 * transmission errors.
	 * 
	 * @return Maximum retries.
	 */
	public int getMaxAdvertisingRetries();

	/**
	 * Limit of contents successfully received by a target device.
	 * 
	 * @return Maximum contents.
	 */
	public int getMaxAdvertisingSuccesses();

	/**
	 * Limit of concurrent transfers per Bluetooth adapter.
	 * 
	 * @return Limit of concurrent transfers.
	 */
	public int getMaxConcurrentTransfers();

	/**
	 * Get the chosen Bluetooth adapter to perform discovery process.
	 * 
	 * @return Bluetooth adapter index, 0 to N.
	 */
	public int getDiscoveryAdapterNumber();

	/**
	 * Interval between checks of new devices that should be advertised.
	 * 
	 * @return Interval of advertising, in milliseconds.
	 */
	public long getAdvertisingInterval();

}
