package br.ime.usp.mac499.bluetooth.common.config;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Structure of recognized options of a Bluetooth parameters file
 * (<code>bluetooth.properties</code>).
 * 
 * @author Rodolpho Atoji
 */
public enum ConfigKey {

	UNKNOWN;
	
	public enum Bluetooth {
		MAX_CONCURRENT		("bluetooth.max_concurrent"),
		DISCOVERY_ADAPTER	("bluetooth.discovery_adapter");
		
		private final String bluetoothString;

		private Bluetooth(String bluetoothString) {
			this.bluetoothString = bluetoothString;
		}

		@Override
		public String toString() {
			return bluetoothString;
		}
	}
	
	public enum Discovery {
		INTERVAL	("discovery.interval");
		
		private final String intervalString;

		private Discovery(String intervalString) {
			this.intervalString = intervalString;
		}

		@Override
		public String toString() {
			return intervalString;
		}
	}
	
	public enum Advertising {
		MAX_RETRIES		("advertising.max_retries"),
		MAX_SUCCESSES	("advertising.max_successes"),
		INTERVAL		("advertising.interval");
		
		private final String advertisingString;

		private Advertising(String advertisingString) {
			this.advertisingString = advertisingString;
		}

		@Override
		public String toString() {
			return advertisingString;
		}
	}
	
}
