package br.ime.usp.mac499.bluetooth.common.discovery;

import java.util.List;

import javax.bluetooth.BluetoothStateException;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Device discovery process specification.
 * 
 * @author Rodolpho Atoji
 */
public interface DeviceDiscoverer {

	/**
	 * Perform synchronized device inquiry, in which the results are
	 * return just at the end of the execution of the process.
	 * 
	 * @param discoveryType General or limited.
	 * @return List of discovered devices in this execution.
	 * @throws BluetoothStateException
	 */
	public abstract List<DiscoveredDevice> doSyncInquiry(
			DiscoveryType discoveryType) throws BluetoothStateException;

	/**
	 * Remove a discovery observer to be notified at the end of a discovery
	 * process.
	 * 
	 * @param observer Discovery observer.
	 */
	public abstract void removeDiscoveryObserver(DiscoveryObserver observer);

	/**
	 * Add a discovery observer to be called whenever a discovery process
	 * ends.
	 * 
	 * @param observer Discovery observer.
	 */
	public abstract void addDiscoveryObserver(DiscoveryObserver observer);

	/**
	 * Get last discovery process status (in progress, failed, success, etc).
	 * 
	 * @return Discovery process status.
	 */
	public abstract DiscoveryStatus getLastStatus();

}