package br.ime.usp.mac499.bluetooth.common.discovery;

import java.util.Date;

import javax.bluetooth.DeviceClass;
import javax.bluetooth.RemoteDevice;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Discovered device in the vicinity by means of the Bluetooth discovery
 * process.
 * 
 * @author Rodolpho Atoji
 */
public class DiscoveredDevice {

	private final RemoteDevice remoteDevice;

	private final DeviceClass deviceClass;

	private final Date timestamp;

	/**
	 * Builds a newly discovered device.
	 * 
	 * @param remoteDevice JSR 82 target device metadata.
	 * @param deviceClass JSR 82 target device class.
	 * @param timestamp Timestamp of discovery.
	 */
	public DiscoveredDevice(RemoteDevice remoteDevice, DeviceClass deviceClass,
			Date timestamp) {
		this.remoteDevice = remoteDevice;
		this.deviceClass = deviceClass;
		this.timestamp = timestamp;
	}

	public RemoteDevice getRemoteDevice() {
		return remoteDevice;
	}

	public DeviceClass getDeviceClass() {
		return deviceClass;
	}

	public Date getTimestamp() {
		return timestamp;
	}

}
