package br.ime.usp.mac499.bluetooth.common.discovery;

import java.util.Date;
import java.util.List;

import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDeviceManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * At the end of a discovery process checks if each discovered device is not
 * known previously and updates its information on the advertising device
 * manager.
 * 
 * @author Rodolpho Atoji
 */
public class DiscoveryResultProcessor implements DiscoveryObserver {

	private AdvertisingDeviceManager deviceManager;

	/**
	 * Build the processor with an advertising device manager to keep
	 * the device scenario updated.
	 * 
	 * @param deviceManager Device manager.
	 */
	public DiscoveryResultProcessor(AdvertisingDeviceManager deviceManager) {
		this.deviceManager = deviceManager;
	}

	@Override
	public void notifyDevicesDiscovered(List<DiscoveredDevice> discoveredList,
			Date startTime, Date endTime, DiscoveryStatus status) {
		processDiscoveredDevices(discoveredList);
	}

	/**
	 * Check if each device is new and/or updates its information on the
	 * advertising device manager.
	 * 
	 * @param discoveredList List of devices found in a device discovery
	 * running.
	 */
	private void processDiscoveredDevices(List<DiscoveredDevice> discoveredList) {
		String currentMacAddress;

		for (DiscoveredDevice device : discoveredList) {
			currentMacAddress = device.getRemoteDevice().getBluetoothAddress();

			if (deviceManager.isNewDevice(currentMacAddress)) {
				AdvertisingDevice advertisingDevice = new AdvertisingDevice(
						device.getRemoteDevice(), device.getDeviceClass(),
						device.getTimestamp());
				deviceManager.addDevice(advertisingDevice);
			} else {
				deviceManager.updateDeviceDiscoveryStatus(currentMacAddress,
						device.getTimestamp());
			}
		}
	}

}
