package br.ime.usp.mac499.bluetooth.common.discovery;

import javax.bluetooth.DiscoveryListener;

/*
 * This file is part of MAC499 Bluetooth NFC 
 * Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Discovery status of a device discovery execution.
 */
public enum DiscoveryStatus {

	/**
	 * The process completed successfully.
	 */
	COMPLETED 	(DiscoveryListener.INQUIRY_COMPLETED),
	
	/**
	 * Error while performing the discovery, results can be not accurate.
	 */
	ERROR		(DiscoveryListener.INQUIRY_ERROR),
	
	/**
	 * Discovery was not completed due to termination request.
	 */
	TERMINATED	(DiscoveryListener.INQUIRY_TERMINATED),
	INVALID		(-1);
	
	private final int discoveryListenerCode;
	
	private DiscoveryStatus(int discoveryListenerCode) {
		this.discoveryListenerCode = discoveryListenerCode;
	}
	
	public int getDiscoveryListenerCode() {
		return discoveryListenerCode;
	}

	/**
	 * Given a JSR 82 discovery status, map it to a 
	 * <code>DiscoveryStatus</code>.
	 * 
	 * @param discoveryListenerCode JSR 82 discovery listener code.
	 * @return Mapped <code>DiscoveryStatus</code>, -1 if mapping is not
	 * 	found.
	 */
	public static DiscoveryStatus getStatusMapping(int discoveryListenerCode) {
		for (DiscoveryStatus status: DiscoveryStatus.values()) {
			if (status.getDiscoveryListenerCode() == discoveryListenerCode)
				return status;
		}

		return INVALID;
	}
	
}
