package br.ime.usp.mac499.bluetooth.common.sdp;

import javax.bluetooth.BluetoothStateException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.ime.usp.mac499.bluetooth.common.BluetoothAdapterMonitor;
import br.ime.usp.mac499.bluetooth.common.BluetoothResources;
import br.ime.usp.mac499.bluetooth.common.stack.LocalAdapterWrapper;
import br.ime.usp.mac499.bluetooth.common.stack.LocalAdaptersManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Thread-wrapper for service discoverer.
 * 
 * @author Rodolpho Atoji
 */
public class RunnableServiceDiscoverer implements Runnable {

	private final ServiceDiscoverer serviceDiscoverer;

	private final int assignedAdapter;

	private final Logger logger;

	/**
	 * Builds a service discovery task to be performed on the selected
	 * adapter.
	 * 
	 * @param serviceDiscoverer Service discoverer.
	 * @param assignedAdapter Selected adapter index.
	 */
	public RunnableServiceDiscoverer(ServiceDiscoverer serviceDiscoverer,
			int assignedAdapter) {
		this.serviceDiscoverer = serviceDiscoverer;
		this.assignedAdapter = assignedAdapter;
		this.logger = LoggerFactory.getLogger(RunnableServiceDiscoverer.class);
	}

	@Override
	public void run() {
		long start = 0;
		String macAddress = serviceDiscoverer.getRemoteDevice()
				.getBluetoothAddress();

		BluetoothAdapterMonitor monitor = BluetoothResources
				.getBluetoothAdapterMonitor();
		LocalAdaptersManager adaptersManager = BluetoothResources
				.getLocalAdaptersManager();

		/* Select natively which Bluetooth adapter to use */
		Object adapter = adaptersManager.getAdapter(assignedAdapter);
		LocalAdapterWrapper adapterWrapper = adaptersManager
				.getLocalAdapterWrapper();

		try {
			adapterWrapper.selectAdapterForUse(adapter);
			monitor.acquireAdapter(assignedAdapter);

			logger.info("SD_STARTED  device: {} adapter: {}", macAddress,
					adapter);
			start = System.currentTimeMillis();

			serviceDiscoverer.doServiceSearch();
		} catch (BluetoothStateException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			ServiceStatus status = serviceDiscoverer.getSdpResult();
			long end = System.currentTimeMillis();
			logger.info("SD_FINISHED device: {} status: {} duration: {}",
					new Object[] { macAddress, status, (end - start) });

			adapterWrapper.releaseUsedAdapter();
			monitor.releaseAdapter(assignedAdapter);
		}
	}

}
