package br.ime.usp.mac499.bluetooth.common.sdp;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Service discovery task.
 * 
 * Given a newly discovered device, a service search discovers its
 * capabilities.
 * 
 * @author Rodolpho Atoji
 */
public interface ServiceDiscoverer {

	/**
	 * Perform service search on the selected target device.
	 * @throws BluetoothStateException
	 */
	public abstract void doServiceSearch() throws BluetoothStateException;

	/**
	 * Get the result of the service search on the device. Blocks until the
	 * service search is complete.
	 * 
	 * @return Status of service search.
	 */
	public abstract ServiceStatus getSdpResult();

	/**
	 * @return Records of the services found on the target device.
	 */
	public abstract ServiceRecord[] getServiceRecords();

	/**
	 * @return JSR 82 metadata about target device.
	 */
	public abstract RemoteDevice getRemoteDevice();

	/**
	 * Remove a service search observer.
	 * 
	 * @param observer Service search observer.
	 */
	public abstract void removeServiceSearchObserver(ServiceSearchObserver observer);

	/**
	 * Add a service search observer, to be notified when the service search
	 * end on the target device.
	 * 
	 * @param observer Service search observer.
	 */
	public abstract void addServiceSearchObserver(ServiceSearchObserver observer);

}