package br.ime.usp.mac499.bluetooth.common.sdp;

import javax.bluetooth.RemoteDevice;
import javax.bluetooth.UUID;

import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Builds service discoverer for single or multiple services searches.
 * 
 * @author Rodolpho Atoji
 */
public class ServiceDiscovererFactory {

	/**
	 * Build a service searcher for a single service.
	 * 
	 * @param device Target device.
	 * @param type Service to search.
	 * @return Service searcher.
	 */
	public static ServiceDiscoverer buildSingleServiceDiscoverer(
			AdvertisingDevice device, ServiceType type) {
		RemoteDevice remoteDevice = device.getRemoteDevice();
		UUID[] uuid = { type.getUUID() };
		return new GenericServiceDiscoverer(null, uuid, remoteDevice);
	}

	/**
	 * Build a service searcher for multiple services.
	 * 
	 * @param device Target device.
	 * @param types Array of services.
	 * @return Service searcher.
	 */
	public static ServiceDiscoverer buildMultipleServiceDiscoverer(
			AdvertisingDevice device, ServiceType[] types) {
		RemoteDevice remoteDevice = device.getRemoteDevice();
		UUID[] uuid = new UUID[types.length];

		for (int i = 0; i < uuid.length; i++)
			uuid[i] = types[i].getUUID();

		return new GenericServiceDiscoverer(null, uuid, remoteDevice);
	}

}
