package br.ime.usp.mac499.bluetooth.common.sdp;

import java.util.Date;

import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Result of service search attempt.
 * 
 * @author Rodolpho Atoji
 */
public class ServiceSearchAttempt {

	private final ServiceStatus result;

	private final RemoteDevice remoteDevice;

	private final ServiceRecord[] serviceRecords;

	private final Date startTime;

	private final Date endTime;

	/**
	 * Construct an attempt with all service search metadata.
	 *  
	 * @param result Service search status.
	 * @param remoteDevice Target device.
	 * @param serviceRecords Service records of services found.
	 * @param startTime Service search start time.
	 * @param endTime Service search end time.
	 */
	public ServiceSearchAttempt(ServiceStatus result,
			RemoteDevice remoteDevice, ServiceRecord[] serviceRecords,
			Date startTime, Date endTime) {
		this.result = result;
		this.remoteDevice = remoteDevice;
		this.serviceRecords = serviceRecords;
		this.startTime = startTime;
		this.endTime = endTime;
	}

	public ServiceStatus getResult() {
		return result;
	}

	public RemoteDevice getRemoteDevice() {
		return remoteDevice;
	}

	public ServiceRecord[] getServiceRecords() {
		return serviceRecords;
	}

	public Date getStartTime() {
		return startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

}
