package br.ime.usp.mac499.bluetooth.common.sdp;

import javax.bluetooth.ServiceRecord;

import br.ime.usp.mac499.bluetooth.model.AdvertisingDeviceManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Processes the result of a service search attempt.
 * 
 * @author Rodolpho Atoji
 */
public class ServiceSearchProcessor implements ServiceSearchObserver {

	private final AdvertisingDeviceManager deviceManager;
	
	/**
	 * Builds a service search processor to update the advertising device
	 * manager.
	 * 
	 * @param deviceManager Advertising device manager.
	 */
	public ServiceSearchProcessor(AdvertisingDeviceManager deviceManager) {
		this.deviceManager = deviceManager;
	}

	@Override
	public void notifySearchResult(ServiceSearchAttempt serviceSearchAttempt) {
		String macAddress = serviceSearchAttempt.getRemoteDevice().getBluetoothAddress();
		ServiceStatus status = serviceSearchAttempt.getResult();
		ServiceRecord serviceRecord[] = serviceSearchAttempt.getServiceRecords();
		
		deviceManager.updateServiceSearchStatus(macAddress, status, serviceRecord);
	}

}
