package br.ime.usp.mac499.bluetooth.common.stack;

import java.util.ArrayList;

import javax.bluetooth.BluetoothStateException;

import com.intel.bluetooth.BlueCoveImpl;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Bluecove-specific local adapters manager.
 * 
 * This class is responsible for detecting and enumerating all attached
 * Bluetooth adapters.
 * 
 * @author Rodolpho Atoji
 */
public class BluecoveLocalAdaptersManager implements LocalAdaptersManager {

	public final static int MAX_SUPPORTED_ADAPTERS = 16;

	public final static int DEFAULT_DISCOVERY_ADAPTER = 0;
	
	private final BluecoveLocalAdapterWrapper wrapper = new BluecoveLocalAdapterWrapper();

	private Object[] adapters;

	@Override
	public void initialize() throws BluetoothStateException {
		System.out.println("Initializing local adapters...");

		Object currentAdapter;
		ArrayList<Object> initializedAdapters = new ArrayList<Object>();
		ArrayList<Object> notFoundAdapters = new ArrayList<Object>();

		for (int i = 0; i < MAX_SUPPORTED_ADAPTERS; i++) {
			try {
				BlueCoveImpl.setConfigProperty("bluecove.deviceID",
						String.valueOf(i));
				BlueCoveImpl.useThreadLocalBluetoothStack();
				currentAdapter = BlueCoveImpl.getThreadBluetoothStackID();

				initializedAdapters.add(currentAdapter);
			} catch (Exception e) {
				notFoundAdapters.add(String.valueOf(i));
			} finally {
				BlueCoveImpl.releaseThreadBluetoothStack();
			}
		}

		printAdapters(initializedAdapters, "Initialized: ");
		printAdapters(notFoundAdapters, "Not found:   ");

		adapters = initializedAdapters.toArray();
		BlueCoveImpl.setThreadBluetoothStackID(null);
	}

	private void printAdapters(ArrayList<Object> adapters, String adaptersString) {
		StringBuffer buffer = new StringBuffer();

		adapters.trimToSize();
		buffer.append(adaptersString);
		for (int i = 0; i < adapters.size(); i++) {
			buffer.append(adapters.get(i));
			if (i < adapters.size() - 1)
				buffer.append(",");
		}

		System.out.println(buffer.toString());
	}

	@Override
	public Object getDiscoveryAdapter() {
		if ((adapters != null) && (adapters.length > 0))
			return adapters[DEFAULT_DISCOVERY_ADAPTER];
		return null;
	}

	@Override
	public int getTotalAdapters() {
		if (adapters != null)
			return adapters.length;
		return 0;
	}

	@Override
	public Object getAdapter(int adapterNumber) {
		if (adapters != null) {
			if ((adapterNumber >= 0) && (adapterNumber < adapters.length))
				return adapters[adapterNumber];
		}
		return null;
	}

	@Override
	public LocalAdapterWrapper getLocalAdapterWrapper() {
		return wrapper;
	}

}
