package br.ime.usp.mac499.bluetooth.db;

import java.util.Date;

import br.ime.usp.mac499.bluetooth.io.FilePusherObserver;
import br.ime.usp.mac499.bluetooth.io.OBEXPushStatus;
import br.ime.usp.mac499.bluetooth.metadata.AdvertisingMetadata;
import br.ime.usp.mac499.bluetooth.model.AdvertisementAttempt;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;
import br.ime.usp.mac499.bluetooth.model.AdvertisingMediaFile;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Persists metadata of a file-pushing attempt, at the end of the 
 * process.
 * 
 * @author Rodolpho Atoji
 */
public class AdvertisingMetadataObserver implements FilePusherObserver {

	private final SimpleDAO simpleDAO;
	
	public AdvertisingMetadataObserver() {
		DAOFactory daoFactory = DAOFactory.getDAOFactory(DAOFactory.JPA);
		this.simpleDAO = daoFactory.getSimpleDAO();
	}
	
	@Override
	public void notifyObserver(AdvertisementAttempt attempt,
			AdvertisingDevice device, AdvertisingMediaFile mediaFile) {
		OBEXPushStatus pushStatus = attempt.getPushStatus();
		String macAddress = device.getMacAddress();
		String filename = mediaFile.getName();
		Date startTime = attempt.getStartTime();
		Date endTime = attempt.getEndTime();

		AdvertisingMetadata metadata = new AdvertisingMetadata(pushStatus,
				macAddress, filename, startTime, endTime);
		simpleDAO.save(metadata);
	}

}
