package br.ime.usp.mac499.bluetooth.db;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Factory of DAOs (Data Access Objects).
 * 
 * Currently only JPA is supported.
 * 
 * @author Rodolpho Atoji
 */
public abstract class DAOFactory {

	public final static int JPA = 0;
	
	private static JPADAOFactory jpadaoFactory;
	
	/**
	 * Get a factory for a selected persistence engine.
	 * 
	 * @param factoryType Currently only JPA is supported.
	 * @return A factory of persistence.
	 */
	public static DAOFactory getDAOFactory(int factoryType) {
		switch (factoryType) {
		case JPA:
			if (jpadaoFactory == null)
				jpadaoFactory = new JPADAOFactory();
			return jpadaoFactory;
		default:
			return null;
		}
	}
	
	/**
	 * @return A DAO which persists all metadata.
	 */
	public abstract SimpleDAO getSimpleDAO();
	
}
