package br.ime.usp.mac499.bluetooth.db;

import java.util.Date;
import java.util.List;

import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveredDevice;
import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveryObserver;
import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveryStatus;
import br.ime.usp.mac499.bluetooth.metadata.DiscoveryMetadata;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Persists discovery metadata at the end of a discovery process.
 * 
 * @author Rodolpho Atoji
 */
public class DiscoveryMetadataObserver implements DiscoveryObserver {

	private final SimpleDAO simpleDAO;

	public DiscoveryMetadataObserver() {
		DAOFactory daoFactory = DAOFactory.getDAOFactory(DAOFactory.JPA);
		this.simpleDAO = daoFactory.getSimpleDAO();
	}

	@Override
	public void notifyDevicesDiscovered(List<DiscoveredDevice> discoveredList,
			Date startTime, Date endTime, DiscoveryStatus status) {
		int discoveredDevices = 0;

		if (discoveredList != null)
			discoveredDevices = discoveredList.size();

		DiscoveryMetadata metadata = new DiscoveryMetadata(startTime, endTime,
				discoveredDevices, status);
		simpleDAO.save(metadata);
	}

}
