package br.ime.usp.mac499.bluetooth.db;

import org.hsqldb.server.Server;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Wrapper for HSQLDB engine.
 * 
 * @author Rodolpho Atoji
 */
public class HSQLDatabaseManager implements DatabaseManager {

	private final String HSQL_DATABASE_NAME = "bluetoothnfc";
	
	private Server hsqlServer;
	
	@Override
	public void open() throws DatabaseException {
		if (hsqlServer != null)
			throw new DatabaseException("Database already open");
		
		String databasePath = getDatabasePath();
		
		this.hsqlServer = new Server();
		hsqlServer.setDatabaseName(0, HSQL_DATABASE_NAME);
		hsqlServer.setDatabasePath(0, databasePath);
		hsqlServer.setAddress("127.0.0.1");
		hsqlServer.setLogWriter(null);

		hsqlServer.start();
	}

	private String getDatabasePath() {
		String baseDir = getBaseDir();
		
		return baseDir + "/database/" + HSQL_DATABASE_NAME;
	}
	
	private String getBaseDir() {
		String basedir = System.getProperty("basedir");

		if (basedir == null)
			basedir = ".";
		return basedir;
	}

	@Override
	public void close() throws DatabaseException {
		if (hsqlServer == null)
			throw new DatabaseException("Database not open");
		
		hsqlServer.stop();
		hsqlServer.shutdown();
	}

}
