package br.ime.usp.mac499.bluetooth.db;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;

import br.ime.usp.mac499.bluetooth.metadata.AdvertisingMetadata;
import br.ime.usp.mac499.bluetooth.metadata.DiscoveryMetadata;
import br.ime.usp.mac499.bluetooth.metadata.ServiceSearchMetadata;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Simple DAO for persisting <code>AdvertisingMetadata</code>,
 * <code>DiscoveryMetadata</code> and <code>ServiceSearchMetadata</code>.
 * 
 * @author Rodolpho Atoji
 */
public class JPASimpleDAO implements SimpleDAO {

	private final EntityManagerFactory factory;
	
	public JPASimpleDAO(EntityManagerFactory factory) {
		this.factory = factory;
	}

	@Override
	public void save(AdvertisingMetadata metadata) {
		persistJPA(metadata);
	}

	@Override
	public void save(DiscoveryMetadata metadata) {
		persistJPA(metadata);
	}

	@Override
	public void save(ServiceSearchMetadata metadata) {
		persistJPA(metadata);
	}
	
	private void persistJPA(Object metadata) {
		EntityManager entityManager = factory.createEntityManager();
		EntityTransaction transaction = entityManager.getTransaction();
		
		try {
			transaction.begin();
			entityManager.persist(metadata);
			transaction.commit();
		} catch (Exception e) {
			if (transaction.isActive())
				transaction.rollback();
		} finally {
			if (entityManager.isOpen())
				entityManager.close();
		}
	}

}
