package br.ime.usp.mac499.bluetooth.db;

import java.util.Date;

import br.ime.usp.mac499.bluetooth.common.sdp.ServiceSearchAttempt;
import br.ime.usp.mac499.bluetooth.common.sdp.ServiceSearchObserver;
import br.ime.usp.mac499.bluetooth.common.sdp.ServiceStatus;
import br.ime.usp.mac499.bluetooth.metadata.ServiceSearchMetadata;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Persists metadata relative to a service search attempt.
 * 
 * @author Rodolpho Atoji
 */
public class ServiceSearchMetadataObserver implements ServiceSearchObserver {

	private final SimpleDAO simpleDAO;
	
	public ServiceSearchMetadataObserver() {
		DAOFactory daoFactory = DAOFactory.getDAOFactory(DAOFactory.JPA);
		this.simpleDAO = daoFactory.getSimpleDAO();
	}
	
	@Override
	public void notifySearchResult(ServiceSearchAttempt serviceSearchAttempt) {
		Date startTime = serviceSearchAttempt.getStartTime();
		Date endTime = serviceSearchAttempt.getEndTime();
		String macAddress = serviceSearchAttempt.getRemoteDevice()
				.getBluetoothAddress();
		ServiceStatus status = serviceSearchAttempt.getResult();

		ServiceSearchMetadata metadata = new ServiceSearchMetadata(startTime,
				endTime, macAddress, status);
		simpleDAO.save(metadata);
	}

}
