package br.ime.usp.mac499.bluetooth.io;

import java.io.IOException;

import br.ime.usp.mac499.bluetooth.db.AdvertisingMetadataObserver;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Specification of an OBEX file pusher, which deliveries multimedia
 * contents to target devices.
 * 
 * @author Rodolpho Atoji
 */
public interface FilePusher {

	/**
	 * Send file to target.
	 * @throws IOException
	 */
	public abstract void pushFile() throws IOException;

	/**
	 * @result Get result of the pushing process.
	 */
	public abstract OBEXPushStatus getPushStatus();

	/**
	 * @return MAC address of the target device.
	 */
	public abstract String getMacAddressDestination();

	/**
	 * @return Filename of the file bing pushed.
	 */
	public abstract String getFilename();

	/**
	 * Prevent a device of receiving multiple transfers simultaneously.
	 * 
	 * @return <code>true</code> if device was successfully locked,
	 * <code>false</code> otherwise.
	 */
	public abstract boolean lockDeviceForTransfer();

	/**
	 * Release a device for transfer.
	 * 
	 * @return <code>true</code> if device was successfully unlocked,
	 * <code>false</code> otherwise.
	 */
	public abstract boolean unlockDeviceForTransfer();

	/**
	 * Add an observer to be notified at the end of the pushing process.
	 * 
	 * @param observer Advertising observer.
	 */
	public abstract void addObserver(AdvertisingMetadataObserver observer);

}