package br.ime.usp.mac499.bluetooth.io;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Results of pushing file attempts.
 * 
 * @author Rodolpho Atoji
 */
public enum OBEXPushStatus {

	SUCCESS			(new String[]{"SUCCESS"}),
	FAILED			(new String[]{"FAILED"}),
	IN_PROGRESS		(new String[]{"IN_PROGRESS"}),
	
	TIMEOUT			(new String[]{"110"}),
	REFUSED			(new String[]{"111", "Session not connected"}),
	UNREACHABLE		(new String[]{"112", "No such device"}),
	UNKNOWN			(new String[]{"UNKNOWN"});
	
	private final String[] stackErrors;
	
	private OBEXPushStatus(String... bluezErrorMessage) {
		this.stackErrors = bluezErrorMessage;
	}
	
	private String[] getErrors() {
		return stackErrors;
	}

	/**
	 * Stack error messages are platform-dependent.
	 * 
	 * This method tries to map platform errors to a standard
	 * <code>OBEXPushStatus</code>.
	 * 
	 * @param stackErrorMessage
	 * @return
	 */
	public static OBEXPushStatus getErrorMapping(String stackErrorMessage) {
		for (OBEXPushStatus status: OBEXPushStatus.values()) {
			for (String currentError: status.getErrors()) {
				if (stackErrorMessage.contains(currentError))
					return status;
			}
		}
		
		return FAILED;
	}

	
}
