package br.ime.usp.mac499.bluetooth.io;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.ime.usp.mac499.bluetooth.common.BluetoothAdapterMonitor;
import br.ime.usp.mac499.bluetooth.common.BluetoothResources;
import br.ime.usp.mac499.bluetooth.common.stack.LocalAdapterWrapper;
import br.ime.usp.mac499.bluetooth.common.stack.LocalAdaptersManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Thread-wrapper for a file pusher.
 * 
 * @author Rodolpho Atoji
 */
public class RunnableFilePusher implements Runnable {

	private final FilePusher filePusher;

	private final int assignedAdapter;

	private final Logger logger;

	/**
	 * Build a file pusher to the assigned adapter.
	 * 
	 * @param filePusher File pusher.
	 * @param assignedAdapter Assigned Bluetooth adapter.
	 */
	public RunnableFilePusher(FilePusher filePusher, int assignedAdapter) {
		this.filePusher = filePusher;
		this.assignedAdapter = assignedAdapter;
		this.logger = LoggerFactory.getLogger(RunnableFilePusher.class);
	}

	@Override
	public void run() {
		long before = 0;
		String macAddress = filePusher.getMacAddressDestination();
		String mediaFilename = filePusher.getFilename();

		BluetoothAdapterMonitor monitor = BluetoothResources
				.getBluetoothAdapterMonitor();
		LocalAdaptersManager adaptersManager = BluetoothResources
				.getLocalAdaptersManager();

		/* Natively select a Bluetooth adapter */
		Object adapter = adaptersManager.getAdapter(assignedAdapter);
		LocalAdapterWrapper adapterWrapper = adaptersManager
				.getLocalAdapterWrapper();
		
		try {
			adapterWrapper.selectAdapterForUse(adapter);
			monitor.acquireAdapter(assignedAdapter);

			logger.info("PUSH_STARTED device: {} adapter: {} mediaFile: {}",
					new Object[] { macAddress, adapter, mediaFilename });
			before = System.currentTimeMillis();

			filePusher.pushFile();
			
			long after = System.currentTimeMillis();
			OBEXPushStatus status = filePusher.getPushStatus();
			logger.info("PUSH_ENDED   device: {} status: {} duration: {}",
					new Object[] { macAddress, status, (after - before) });
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			filePusher.unlockDeviceForTransfer();
			adapterWrapper.releaseUsedAdapter();
			monitor.releaseAdapter(assignedAdapter);
		}
	}

}
