package br.ime.usp.mac499.bluetooth.metadata;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import br.ime.usp.mac499.bluetooth.io.OBEXPushStatus;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Metadata about a single advertising attempt.
 * 
 * @author Rodolpho Atoji
 */
@Entity
@Table(name = "ADVERTISINGMETADATA")
public class AdvertisingMetadata {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "ID")
	private Long id;

	@Enumerated(EnumType.STRING)
	@Column(name = "PUSHSTATUS", length = 32, nullable = false)
	private OBEXPushStatus pushStatus;

	@Column(name = "MACADDRESS", length = 12, nullable = false)
	private String macAddress;

	@Column(name = "FILENAME", nullable = false)
	private String filename;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "STARTTIME", nullable = false)
	private Date startTime;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "ENDTIME", nullable = false)
	private Date endTime;

	public AdvertisingMetadata() {

	}

	public AdvertisingMetadata(OBEXPushStatus pushStatus, String macAddress,
			String filename, Date startTime, Date endTime) {
		super();
		this.pushStatus = pushStatus;
		this.macAddress = macAddress;
		this.filename = filename;
		this.startTime = startTime;
		this.endTime = endTime;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public OBEXPushStatus getPushStatus() {
		return pushStatus;
	}

	public void setPushStatus(OBEXPushStatus pushStatus) {
		this.pushStatus = pushStatus;
	}

	public String getMacAddress() {
		return macAddress;
	}

	public void setMacAddress(String macAddress) {
		this.macAddress = macAddress;
	}

	public String getFilename() {
		return filename;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

}
