package br.ime.usp.mac499.bluetooth.metadata;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveryStatus;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Metadata about each discovery process performed.
 * 
 * @author Rodolpho Atoji
 */
@Entity
@Table(name = "DISCOVERYMETADATA")
public class DiscoveryMetadata {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "ID")
	private Long id;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "STARTTIME", nullable = false)
	private Date startTime;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "ENDTIME", nullable = false)
	private Date endTime;

	@Column(name = "DISCOVEREDDEVICES", nullable = false)
	private int discoveredDevices;

	@Enumerated(EnumType.STRING)
	@Column(name = "STATUS", length = 32, nullable = false)
	private DiscoveryStatus status;

	public DiscoveryMetadata() {

	}

	public DiscoveryMetadata(Date startTime, Date endTime,
			int discoveredDevices, DiscoveryStatus status) {
		super();
		this.startTime = startTime;
		this.endTime = endTime;
		this.discoveredDevices = discoveredDevices;
		this.status = status;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public int getDiscoveredDevices() {
		return discoveredDevices;
	}

	public void setDiscoveredDevices(int discoveredDevices) {
		this.discoveredDevices = discoveredDevices;
	}

	public DiscoveryStatus getStatus() {
		return status;
	}

	public void setStatus(DiscoveryStatus status) {
		this.status = status;
	}

}
