package br.ime.usp.mac499.bluetooth.model;

import java.util.Date;

import br.ime.usp.mac499.bluetooth.io.OBEXPushStatus;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Records an advertisement attempt to a Bluetooth device.
 * 
 * The attempts are checked to determine if a target device should still
 * receive new advertisements.
 * 
 * @author Rodolpho Atoji
 */
public class AdvertisementAttempt {

	private final Date startTime;

	private final Date endTime;

	private final AdvertisingMediaFile mediaFile;

	private final OBEXPushStatus pushStatus;

	public AdvertisementAttempt(Date startTime, Date endTime,
			AdvertisingMediaFile mediaFile, OBEXPushStatus pushStatus) {
		this.startTime = startTime;
		this.endTime = endTime;
		this.mediaFile = mediaFile;
		this.pushStatus = pushStatus;
	}

	public AdvertisingMediaFile getMediaFile() {
		return mediaFile;
	}

	public OBEXPushStatus getPushStatus() {
		return pushStatus;
	}

	public Date getStartTime() {
		return startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

}
