package br.ime.usp.mac499.bluetooth.model;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Multimedia content to deliver to an advertising device.
 * 
 * @author Rodolpho Atoji
 */
public class AdvertisingMediaFile {

	/**
	 * Filename.
	 */
	private final String name;

	/**
	 * MIME type of the content.
	 */
	private final MediaType mimeType;

	/**
	 * RAW content loaded from storage.
	 */
	private final byte[] content;

	public AdvertisingMediaFile(String name, MediaType mediaType, byte[] content) {
		this.name = name;
		this.mimeType = mediaType;
		this.content = content;
	}

	public String getName() {
		return name;
	}

	public String getMimeType() {
		return mimeType.getMimeType();
	}

	public byte[] getContent() {
		return content;
	}

	public long getContentSize() {
		return content.length;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof AdvertisingMediaFile))
			return false;

		AdvertisingMediaFile otherMediaFile = (AdvertisingMediaFile) obj;

		return (name.equals(otherMediaFile.name)
				&& (mimeType.equals(otherMediaFile.mimeType)) && (content
				.equals(otherMediaFile.content)));
	}

}
