package br.ime.usp.mac499.bluetooth.model;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Default policy for ordering advertising devices.
 * 
 * NFC-enabled devices has the highest priority.
 * 
 * @author Rodolpho Atoji
 */
public class DefaultPriorityPolicy implements PriorityPolicy {

	private final Priority[] priorities;

	public DefaultPriorityPolicy(Priority... orderedPriority) {
		this.priorities = orderedPriority;
	}

	@Override
	public int compare(AdvertisingDevice o1, AdvertisingDevice o2) {
		for (Priority currentPriority : priorities) {
			switch (currentPriority) {
			case NFC_INITIATED:
				if (o1.isNfcInitiated() && o2.isNfcInitiated())
					return 0;
				else if (o1.isNfcInitiated() && !o2.isNfcInitiated())
					return -1;
				else if (!o1.isNfcInitiated() && o2.isNfcInitiated())
					return 1;
				break;
			case WHEN_DISCOVERED:
				if (o1.getLastDiscoveredTimestamp().after(o2.getLastDiscoveredTimestamp()))
					return -1;
				else if (o1.getLastDiscoveredTimestamp().before(o2.getLastDiscoveredTimestamp()))
					return 1;
				break;
			case ADVERTISING_ERROR:
				if (o1.getFailedAdvertisementAttempts() < o2.getFailedAdvertisementAttempts())
					return -1;
				else if (o1.getFailedAdvertisementAttempts() > o2.getFailedAdvertisementAttempts())
					return 1;
				break;
			case ADVERTISING_SUCCESS:
				if (o1.getSuccesfulAdvertisementAttempts() < o2.getSuccesfulAdvertisementAttempts())
					return -1;
				else if (o1.getSuccesfulAdvertisementAttempts() > o2.getSuccesfulAdvertisementAttempts())
					return 1;
				break;
			case SERVICE_SEARCH_ERROR:
				if (o1.getServiceSearchFailures() < o2.getServiceSearchFailures())
					return -1;
				else if (o1.getServiceSearchFailures() > o2.getServiceSearchFailures())
					return 1;
				break;
			case TIMES_DISCOVERED:
				if (o1.getTimesDiscovered() < o2.getTimesDiscovered())
					return 1;
				else if (o1.getTimesDiscovered() > o2.getTimesDiscovered())
					return -1;				
				break;
			default:
				break;
			}
		}
		
		return 0;
	}
}
