package br.ime.usp.mac499.bluetooth.model;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Media manager which delivers a single and fixed text message.
 * 
 * @author Rodolpho Atoji
 */
public class FixedTextMediaManager implements MediaManager {

	private final AdvertisingMediaFile fixedTextMediaFile;

	/**
	 * Constructs a media file with the specified text.
	 * 
	 * @param fixedText Text to deliver.
	 */
	public FixedTextMediaManager(String fixedText) {
		this.fixedTextMediaFile = new AdvertisingMediaFile("fixed.txt",
				MediaType.TEXT_PLAIN, fixedText.getBytes());
	}

	@Override
	public void reloadMedia(String mediaPath) {
		return;
	}

	@Override
	public AdvertisingMediaFile getNextMediaForDevice(AdvertisingDevice device) {
		if (device.hasAlreadyBeenAdvertisedWith(fixedTextMediaFile))
			return null;
		
		return fixedTextMediaFile;
	}

}
