package br.ime.usp.mac499.bluetooth.model;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Media managers are responsible for the contents which will be
 * delivered to the advertising devices.
 * 
 * Each implementation can choose between delivering multiple or single
 * contents, different media types, texts etc.
 * 
 * @author Rodolpho Atoji
 */
public interface MediaManager {

	/**
	 * Load or reloads the media from storage.
	 * 
	 * Should be called if the file was changed on storage or if
	 * initializing the manager.
	 * 
	 * @param mediaPath
	 */
	public abstract void reloadMedia(String mediaPath);

	/**
	 * If there is more than one content, return the next after the
	 * last content delivered.
	 * 
	 * @param device Target device.
	 * @return Next advertising media, or <code>null</code> if all the
	 * contents available were delivered.
	 */
	public abstract AdvertisingMediaFile getNextMediaForDevice(
			AdvertisingDevice device);

}