package br.ime.usp.mac499.bluetooth.model;

import java.util.LinkedList;
import java.util.List;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Support for delivering multiple media files, sequentially.
 * 
 * Multiple calls to <code>reloadMedia()</code> with different contents
 * append them to a contents list.
 * 
 * @author Rodolpho Atoji
 */
public class MultimediaMultipleFileManager extends MultimediaSingleFileManager {

	private List<AdvertisingMediaFile> mediaFileList;

	public MultimediaMultipleFileManager() {
		this.mediaFileList = new LinkedList<AdvertisingMediaFile>();
	}
	
	@Override
	public void reloadMedia(String mediaPath) {
		AdvertisingMediaFile mediaFile = loadFileFromDisk(mediaPath);
		
		AdvertisingMediaFile mediaFileToReload = null;
		if (mediaFile != null) {
			for (AdvertisingMediaFile previousMedia : mediaFileList) {
				if (previousMedia.getName().equals(mediaFile.getName())) {
					mediaFileToReload = previousMedia;
					break;
				}
			}
			
			if (mediaFileToReload != null) {
				int positionToReplace = mediaFileList.indexOf(mediaFileToReload);
				mediaFileList.set(positionToReplace, mediaFile);
			} else {
				mediaFileList.add(mediaFile);
			}
		}
	}

	@Override
	public AdvertisingMediaFile getNextMediaForDevice(AdvertisingDevice device) {
		for (AdvertisingMediaFile mediaFile : mediaFileList) {
			if (!device.hasAlreadyBeenAdvertisedWith(mediaFile))
				return mediaFile;
		}
		
		return null;
	}
	
	
	
}
