package br.ime.usp.mac499.bluetooth.model;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import br.ime.usp.mac499.util.MimeUtils;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Single multimedia file delivery.
 * 
 * Multiple calls to <code>reloadMedia()</code> causes the manager to
 * replace the current managed content.
 * 
 * @author Rodolpho Atoji
 */
public class MultimediaSingleFileManager implements MediaManager {

	private AdvertisingMediaFile mediaFile;

	@Override
	public void reloadMedia(String mediaPath) {
		this.mediaFile = loadFileFromDisk(mediaPath);
	}

	@Override
	public AdvertisingMediaFile getNextMediaForDevice(AdvertisingDevice device) {
		if (device.hasAlreadyBeenAdvertisedWith(mediaFile))
			return null;
		
		return mediaFile;
	}

	protected AdvertisingMediaFile loadFileFromDisk(String mediaPath) {
		File mediaFile = new File(mediaPath);

		try {
			FileInputStream fileInputStream = new FileInputStream(mediaFile);
			BufferedInputStream bufferedInputStream = new BufferedInputStream(
					fileInputStream);

			byte[] fileBytes = new byte[fileInputStream.available()];
			bufferedInputStream.read(fileBytes);
			bufferedInputStream.close();

			fileInputStream = new FileInputStream(mediaFile);
			bufferedInputStream = new BufferedInputStream(fileInputStream);

			MediaType mediaType = MimeUtils.getMediaTypeOf(mediaPath);
			
			bufferedInputStream.close();

			AdvertisingMediaFile advertisingMediaFile = new AdvertisingMediaFile(
					mediaFile.getName(), mediaType, fileBytes);

			return advertisingMediaFile;
		} catch (IOException e) {
			return null;
		}
	}

}
