package br.ime.usp.mac499.nfc;

import java.util.Date;

import javax.microedition.contactless.ndef.NDEFMessage;
import javax.microedition.contactless.ndef.NDEFRecord;
import javax.microedition.contactless.ndef.NDEFRecordListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.ime.usp.mac499.bluetooth.common.sdp.ServiceStatus;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDeviceManager;
import br.ime.usp.mac499.util.StringUtils;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Listener for NFC touch events.
 * 
 * When an event is detected, the NFC-enabled device is added to the
 * advertising device manager.
 * 
 * @author Rodolpho Atoji
 */
public class NFCDeviceListener implements NDEFRecordListener {

	private final AdvertisingDeviceManager deviceManager;

	private final Logger logger;

	public NFCDeviceListener(AdvertisingDeviceManager deviceManager) {
		this.deviceManager = deviceManager;
		this.logger = LoggerFactory.getLogger(NFCDeviceListener.class);
	}

	@Override
	public void recordDetected(NDEFMessage ndefMessage) {
		if (ndefMessage != null) {
			NDEFRecord[] records = ndefMessage.getRecords();
			if (records != null) {
				String btConnectionURL = new String(records[0].getPayload());
				addDeviceToDiscoveredSet(btConnectionURL);

				logger.info("NFC_READ_OK     device: {} url: {}",
						btConnectionURL);
			} else 
				logger.info("NFC_READ_FAILED device: {}");
		} else {
			logger.error("NFC_NULL_NDEF");
		}
	}

	private void addDeviceToDiscoveredSet(String btConnectionURL) {
		String macAddress = StringUtils
				.extractMacAddressFromOBEXConnectionURL(btConnectionURL);
		if (macAddress == null) {
			logger.error("NFC_BAD_URL     url: {}", btConnectionURL);
			return;
		}

		if (deviceManager.isNewDevice(macAddress))
			insertDeviceIntoManager(macAddress, btConnectionURL);
		else
			updatePushStatus(macAddress, btConnectionURL);
	}

	private void updatePushStatus(String macAddress, String btConnectionURL) {
		deviceManager.setServiceSearchStatus(macAddress,
				ServiceStatus.COMPLETED, btConnectionURL);
	}

	private void insertDeviceIntoManager(String macAddress,
			String btConnectionURL) {
		Date now = new Date();
		AdvertisingDevice device = new AdvertisingDevice(macAddress,
				btConnectionURL, now);

		deviceManager.addDevice(device);
		updatePushStatus(macAddress, btConnectionURL);
	}

}
