package br.ime.usp.mac499.nfc;

import javax.microedition.contactless.ContactlessException;
import javax.microedition.contactless.DiscoveryManager;
import javax.microedition.contactless.ndef.NDEFRecordType;

import br.ime.usp.mac499.bluetooth.common.BluetoothResources;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDeviceManager;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * This class is responsible for starting and stopping the NFC
 * discovery manager.
 * 
 * Once started the NFC discovery manager, the system is able to
 * detect NFC touches.
 * 
 * @author Rodolpho Atoji
 */
public class NFCDeviceManager {

	private DiscoveryManager discoveryManager;

	private final NFCDeviceListener nfcDeviceListener;

	private final NDEFRecordType ndefRecordType;

	public NFCDeviceManager() {
		AdvertisingDeviceManager deviceManager = BluetoothResources
				.getAdvertisingDeviceManager();

		this.discoveryManager = DiscoveryManager.getInstance();
		this.nfcDeviceListener = new NFCDeviceListener(deviceManager);
		this.ndefRecordType = new NDEFRecordType(NDEFRecordType.MIME,
				"text/plain");
	}

	public void start() {
		try {
			discoveryManager.addNDEFRecordListener(nfcDeviceListener,
					ndefRecordType);
		} catch (ContactlessException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void stop() {
		discoveryManager.removeNDEFRecordListener(nfcDeviceListener,
				ndefRecordType);
	}

}
