package br.ime.usp.mac499.util;

import java.util.Collection;

import eu.medsea.mimeutil.MimeType;
import eu.medsea.mimeutil.MimeUtil;
import br.ime.usp.mac499.bluetooth.model.MediaType;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Utility class to guess MIME types from any file.
 * 
 * @author Rodolpho Atoji
 */
public class MimeUtils {
	static {
		registerDetector();
	}

	/**
	 * Given a file, returns the guessing of its MIME type.
	 * 
	 * @param filePath Path of file.
	 * @return Guessed MIME type.
	 */
	@SuppressWarnings("unchecked")
	public static MediaType getMediaTypeOf(String filePath) {
		Collection<MimeType> mimeTypesDetected = MimeUtil
				.getMimeTypes(filePath);

		if (mimeTypesDetected.size() == 0)
			return MediaType.UNSUPPORTED;

		MimeType[] detectedArray = new MimeType[mimeTypesDetected.size()];
		detectedArray = mimeTypesDetected.toArray(detectedArray);

		String mimeType = detectedArray[0].getMediaType() + "/"
				+ detectedArray[0].getSubType();
		return MediaType.getMappingFor(mimeType);
	}

	private static void registerDetector() {
		if (SystemUtils.isLinux())
			MimeUtil.registerMimeDetector("eu.medsea.mimeutil.detector.OpendesktopMimeDetector");
		else if (SystemUtils.isWindows())
			MimeUtil.registerMimeDetector("eu.medsea.mimeutil.detector.WindowsRegistryMimeDetector");
	}

}
