package br.ime.usp.mac499.util;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Utility class to manage the location of configuration files and paths.
 * 
 * @author Rodolpho Atoji
 */
public class PathUtils {

	private static final String DEVICE_FILTER_FILE = "filter.properties";

	private static final String CONFIG_FILE = "bluetooth.properties";
	
	/**
	 * @return Absolute location of <code>bluetooth.properties</code>.
	 */
	public static String getConfigPath() {
		String basedir = getBaseDir();
		String configDir = getConfigDir();

		return basedir + "/" + configDir + "/" + CONFIG_FILE;
	}

	private static String getConfigDir() {
		String configDir = System.getProperty("configDir");

		if (configDir == null)
			configDir = "config";
		return configDir;
	}

	private static String getBaseDir() {
		String basedir = System.getProperty("basedir");

		if (basedir == null)
			basedir = ".";
		return basedir;
	}
	
	/**
	 * @return Absolute location of <code>filter.properties</code>.
	 */
	public static String getDeviceFilterPath() {
		String basedir = getBaseDir();
		String configDir = getConfigDir();

		return basedir + "/" + configDir + "/" + DEVICE_FILTER_FILE;
	}

}
