package br.ime.usp.mac499.util;

import java.util.List;

import br.ime.usp.mac499.bluetooth.common.discovery.DiscoveredDevice;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;

/*
 * This file is part of MAC499 Bluetooth NFC Copyright (C) 2010 Rodolpho Atoji
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/**
 * Utility class to perform some string operations.
 * 
 * @author Rodolpho Atoji 
 */
public class StringUtils {

	/**
	 * Format a list of discovered devices into a DSV string.
	 * 
	 * @param deviceList Discovered device list.
	 * @param separator Delimiter.
	 * @return DSV string.
	 */
	public static String getSeparatedListOfDiscoveredDevices(
			List<DiscoveredDevice> deviceList, String separator) {
		StringBuffer buffer = new StringBuffer();

		for (DiscoveredDevice device : deviceList) {
			buffer.append(device.getRemoteDevice().getBluetoothAddress())
					.append(separator);
		}

		if (deviceList.size() > 0) {
			buffer.trimToSize();
			buffer.deleteCharAt(buffer.length() - 1);
		}

		return buffer.toString();
	}

	/**
	 * Format a list of advertising devices into a DSV string.
	 * 
	 * @param deviceList Advertising device list.
	 * @param separator Delimiter.
	 * @return DSV string.
	 */
	public static String getSeparatedListOfAdvertisingDevices(
			List<AdvertisingDevice> deviceList, String separator) {
		StringBuffer buffer = new StringBuffer();

		for (AdvertisingDevice device : deviceList)
			buffer.append(device.getMacAddress()).append(separator);

		if (deviceList.size() > 0) {
			buffer.trimToSize();
			buffer.deleteCharAt(buffer.length() - 1);
		}

		return buffer.toString();
	}

	/**
	 * Extract the MAC address from a JSR 82 connection URL.
	 * 
	 * @param connectionURL JSR 82 connection URL.
	 * @return MAC address.
	 */
	public static String extractMacAddressFromOBEXConnectionURL(
			final String connectionURL) {
		String obexIdentifier = "btgoep://";

		if (connectionURL.startsWith("btgoep://")) {
			int start = obexIdentifier.length();
			int end = start + 12;
			String macAddress = connectionURL.substring(start, end);

			return macAddress;
		}

		return null;
	}
}
