package br.ime.usp.mac499.test.bluetooth.advertising;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import br.ime.usp.mac499.bluetooth.advertising.AdvertisingDeviceAddressFilter;
import br.ime.usp.mac499.bluetooth.advertising.AdvertisingDeviceFilterChain;
import br.ime.usp.mac499.bluetooth.model.AdvertisingDevice;

public class AdvertisingDeviceAddressFilterTest {

	private static AdvertisingDevice advertisingDevice11;

	private static AdvertisingDevice advertisingDevice22;

	private static AdvertisingDevice advertisingDevice33;

	private static AdvertisingDevice advertisingDevice44;

	private static AdvertisingDevice advertisingDevice88;

	private static AdvertisingDevice advertisingDevice99;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		advertisingDevice11 = new AdvertisingDevice("000000000011", null, null);
		advertisingDevice22 = new AdvertisingDevice("000000000022", null, null);
		advertisingDevice33 = new AdvertisingDevice("000000000033", null, null);
		advertisingDevice44 = new AdvertisingDevice("000000000044", null, null);
		advertisingDevice88 = new AdvertisingDevice("000000000088", null, null);
		advertisingDevice99 = new AdvertisingDevice("000000000099", null, null);
	}

	@Test
	public void testFileFilterNotFound() {
		AdvertisingDeviceAddressFilter singleFilter = new AdvertisingDeviceAddressFilter(
				"invalidfile.properties");

		boolean doFilter = singleFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice44);
		assertFalse(doFilter);
	}

	@Test
	public void testFileFilterNotFoundComposite() {
		AdvertisingDeviceAddressFilter compositeFilter = new AdvertisingDeviceAddressFilter(
				"invalidfile.properties");
		compositeFilter.setNext(new DoNotFilterOut());

		AdvertisingDeviceAddressFilter compositeFilterOut = new AdvertisingDeviceAddressFilter(
				"invalidfile.properties");
		compositeFilterOut.setNext(new FilterOut());

		boolean doFilter = compositeFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);
		doFilter = compositeFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = compositeFilter.shouldFilter(advertisingDevice44);
		assertFalse(doFilter);
		doFilter = compositeFilterOut.shouldFilter(advertisingDevice44);
		assertTrue(doFilter);
	}

	@Test
	public void testDoFilterSingleFilter() {
		AdvertisingDeviceAddressFilter singleFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_do_filter.properties");

		boolean doFilter = singleFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice22);
		assertFalse(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice33);
		assertTrue(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice44);
		assertTrue(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice88);
		assertTrue(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice99);
		assertTrue(doFilter);
	}

	@Test
	public void testDoFilterCompositeFilter() {
		AdvertisingDeviceAddressFilter compositeFilterOut = new AdvertisingDeviceAddressFilter(
				"test/config/filter_do_filter.properties");
		compositeFilterOut.setNext(new FilterOut());

		AdvertisingDeviceAddressFilter compositeDoNotFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_do_filter.properties");
		compositeDoNotFilter.setNext(new DoNotFilterOut());

		boolean doFilter = compositeFilterOut.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice22);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice22);
		assertFalse(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice33);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice33);
		assertTrue(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice44);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice44);
		assertTrue(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice88);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice88);
		assertTrue(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice99);
		assertTrue(doFilter);
		doFilter = compositeDoNotFilter.shouldFilter(advertisingDevice99);
		assertTrue(doFilter);
	}

	@Test
	public void testAllowAllSingleFilter() {
		AdvertisingDeviceAddressFilter singleFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all.properties");

		boolean doFilter = singleFilter.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice33);
		assertFalse(doFilter);

		AdvertisingDeviceAddressFilter singleFilterWithDevices = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all_withdevices.properties");

		doFilter = singleFilterWithDevices.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		doFilter = singleFilterWithDevices.shouldFilter(advertisingDevice33);
		assertFalse(doFilter);
	}

	@Test
	public void testAllowAllCompositeFilter() {
		AdvertisingDeviceAddressFilter compositeFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all.properties");
		compositeFilter.setNext(new FilterOut());

		AdvertisingDeviceAddressFilter compositeFilterOut = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all.properties");
		compositeFilterOut.setNext(new DoNotFilterOut());

		boolean doFilter = compositeFilter.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		doFilter = compositeFilterOut.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

		AdvertisingDeviceAddressFilter compositeFilterWithDevices = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all_withdevices.properties");
		compositeFilterWithDevices.setNext(new FilterOut());

		AdvertisingDeviceAddressFilter compositeFilterOutWithDevices = new AdvertisingDeviceAddressFilter(
				"test/config/filter_allow_all_withdevices.properties");
		compositeFilterOutWithDevices.setNext(new DoNotFilterOut());

		doFilter = compositeFilterWithDevices.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		doFilter = compositeFilterOutWithDevices
				.shouldFilter(advertisingDevice11);
		assertFalse(doFilter);

	}

	@Test
	public void testDenyAllSingleFilter() {
		AdvertisingDeviceAddressFilter singleFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all.properties");

		boolean doFilter = singleFilter.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		doFilter = singleFilter.shouldFilter(advertisingDevice33);
		assertTrue(doFilter);

		AdvertisingDeviceAddressFilter singleFilterWithDevice = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all_withdevices.properties");

		doFilter = singleFilterWithDevice.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		doFilter = singleFilterWithDevice.shouldFilter(advertisingDevice33);
		assertTrue(doFilter);
	}

	@Test
	public void testDenyAllCompositeFilter() {
		AdvertisingDeviceAddressFilter compositeFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all.properties");
		compositeFilter.setNext(new FilterOut());

		boolean doFilter = compositeFilter.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		compositeFilter = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all.properties");
		compositeFilter.setNext(new DoNotFilterOut());

		doFilter = compositeFilter.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		AdvertisingDeviceAddressFilter compositeFilterWithDevices = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all_withdevices.properties");
		compositeFilterWithDevices.setNext(new FilterOut());

		doFilter = compositeFilterWithDevices.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);

		compositeFilterWithDevices = new AdvertisingDeviceAddressFilter(
				"test/config/filter_deny_all_withdevices.properties");
		compositeFilterWithDevices.setNext(new DoNotFilterOut());

		doFilter = compositeFilterWithDevices.shouldFilter(advertisingDevice11);
		assertTrue(doFilter);
	}

	private final class FilterOut implements AdvertisingDeviceFilterChain {

		@Override
		public void setNext(AdvertisingDeviceFilterChain processor) {
		}

		@Override
		public boolean shouldFilter(AdvertisingDevice device) {
			return true;
		}

	}

	private final class DoNotFilterOut implements AdvertisingDeviceFilterChain {

		@Override
		public void setNext(AdvertisingDeviceFilterChain processor) {
		}

		@Override
		public boolean shouldFilter(AdvertisingDevice device) {
			return false;
		}

	}

}
